"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const stream_1 = require("stream");
class Rank extends stream_1.PassThrough {
    constructor(handler) {
        super({ objectMode: true });
        this.accumulator = {};
        this.handler = handler;
        this.on('data', (data) => {
            const acc = this.accumulator;
            const key = this.handler(data);
            acc[key] ? acc[key]++ : (acc[key] = 1);
        });
    }
    resultObject(ascending = false) {
        return lodash_1.fromPairs(this.result(ascending));
    }
    resultArray(ascending = false) {
        const list = this.result(ascending);
        return list.map(([k, v]) => ({ [k]: v })).reverse();
    }
    result(ascending = false) {
        const pairs = lodash_1.toPairs(this.accumulator);
        const sorted = lodash_1.sortBy(pairs, p => p[1]);
        return ascending ? sorted.reverse() : sorted;
    }
}
exports.Rank = Rank;
//# sourceMappingURL=rank.js.map