"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const stream_1 = require("stream");
class JsonFile extends stream_1.Writable {
    constructor(filePath, pretty = false) {
        super({ objectMode: true });
        this.pretty = pretty;
        this.firstLineWritten = false;
        this.writtenLines = 0;
        this.writtenBytes = 2;
        this.fileStream = fs_1.createWriteStream(filePath);
        this.fileStream.write('[');
        this.on('finish', () => this.fileStream.write(']'));
    }
    _write(chunk, encoding, callback) {
        const json = this.pretty
            ? JSON.stringify(chunk, null, 2)
            : JSON.stringify(chunk);
        const prefix = this.firstLineWritten ? ',\n' : '';
        const str = prefix + json;
        this.fileStream.write(str);
        this.firstLineWritten = true;
        this.writtenLines++;
        this.writtenBytes += Buffer.byteLength(str, 'utf-8');
        callback();
    }
    promise() {
        return new Promise((resolve, reject) => {
            this.on('error', error => reject(error));
            this.on('finish', () => {
                const result = { lines: this.writtenLines, bytes: this.writtenBytes };
                resolve(result);
            });
        });
    }
}
exports.JsonFile = JsonFile;
//# sourceMappingURL=json_file.js.map