"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
const aws_1 = require("../../aws");
class CloudWatchLog extends stream_1.Readable {
    constructor(logGroupName, startTime, endTime) {
        super({ objectMode: true });
        this.nextToken = undefined;
        this.buffer = [];
        this.hasStarted = false;
        this.logGroupName = logGroupName;
        this.startTime = startTime || undefined;
        this.endTime = endTime || undefined;
        this.client = new aws_1.default.CloudWatchLogs();
    }
    _read() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.buffer.length) {
                this.push(this.buffer.shift());
            }
            else if (!this.hasStarted || this.nextToken) {
                this.hasStarted = true;
                const entries = yield this.fetchPage();
                this.buffer = entries.events;
                this.nextToken = entries.nextToken;
                this._read();
            }
            else {
                this.push(null);
            }
        });
    }
    fetchPage() {
        return __awaiter(this, void 0, void 0, function* () {
            const logGroupName = this.logGroupName;
            const params = { logGroupName };
            params.startTime = this.startTime;
            params.endTime = this.endTime;
            params.nextToken = this.nextToken;
            return this.client.filterLogEvents(params).promise();
        });
    }
}
exports.CloudWatchLog = CloudWatchLog;
//# sourceMappingURL=cloud_watch_log.js.map