"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const transform_object_data_1 = require("./transform_object_data");
function parseReportLine(str) {
    const rx = /REPORT RequestId: .*\tDuration: (.+) ms\tBilled Duration: (.+) ms \tMemory Size: (.+) MB\tMax Memory Used: (.+) MB/;
    const [, duration, billedDuration, maxMemory, memoryUsed] = str.match(rx);
    return {
        billedDuration: parseFloat(billedDuration),
        duration: parseFloat(duration),
        maxMemory: parseFloat(maxMemory),
        memoryUsed: parseFloat(memoryUsed),
    };
}
function bufferToLog(buffer) {
    const startEvent = buffer.shift();
    const reportEvent = buffer.pop();
    const endEvent = buffer.pop();
    return {
        endTime: endEvent.timestamp,
        messages: buffer.map(e => e.message),
        performance: parseReportLine(reportEvent.message),
        startTime: startEvent.timestamp,
    };
}
class ParseLambdaLog extends transform_object_data_1.TransformObjectData {
    constructor() {
        super();
        this.handler = (data) => {
            if (!this.buffer.length && !data.message.startsWith('START')) {
                return;
            }
            else if (!data.message.startsWith('REPORT')) {
                this.buffer.push(data);
                return;
            }
            else {
                this.buffer.push(data);
                const result = bufferToLog(this.buffer);
                this.buffer = [];
                return result;
            }
        };
    }
}
exports.ParseLambdaLog = ParseLambdaLog;
//# sourceMappingURL=parse_lambda_log.js.map